"""
These test cases are designed to exercise the validation stage of the
pipeline.
"""
from code.src.summoner.configs import load_config, process_config

EXAMPLE_STATE_S3_BUCKET = "ifrit-summoner-test-statefiles"

def test_configs_example_file():
    """
    Test the example config file correctly loads into
    a configuration dictionary.
    """
    config = load_config(config_file='files/example-config.toml')
    assert config['summoner']['env_gitlab_api_token'] == 'test_token'
    assert config['summoner']['env_pagerduty_key_low'] == 'low_key'
    assert config['summoner']['env_pagerduty_key_high'] == 'high_key'
    assert config['aws-account'][0]['terraform']['state']['s3_path'] == '/exampleproject/test/tf.state'

def test_configs_minimal_file():
    """
    Test an example of a config file with the minimal
    number of configurations.  This should excerise our
    default settings.
    """
    config = load_config(config_file='files/minimal-example.toml')
    assert config['aws-account'][0]['terraform']['state']['s3_path'] == '/exampleproject/test/tf.state'
    assert config['aws-account'][0]['terraform']['state']['s3_region'] == 'us-east-1'

def test_configs_various_states():
    """
    Test an example of a config file with the minimal
    number of configurations, and then add in some settings manually.
    This will test when some of the options (but not all) are given.
    Our code should fill in the rest of the defaults in these cases.
    """

    # test that the state dictionary is created
    # if only the terraform dictionary is in the toml
    config_file = 'files/minimal-example.toml'
    config = load_config(config_file, o_validate_configs=False)
    config['aws-account'][0]['terraform'] = dict()
    config = process_config(config)
    assert config['aws-account'][0]['terraform']['state']['s3_path'] == '/exampleproject/test/tf.state'
    assert config['aws-account'][0]['terraform']['state']['s3_region'] == 'us-east-1'

    def load_minimal(config_file):
        """
        Quick inline helper function to reduce code duplication
        as we reuse a bare config structure
        """
        config = load_config(config_file=config_file, o_validate_configs=False)
        config['aws-account'][0]['terraform'] = dict()
        config['aws-account'][0]['terraform']['state'] = dict()
        return config

    # test when someone put the dict in toml, but no settings in the dictionary
    config = load_minimal(config_file)
    config = process_config(config)
    assert config['aws-account'][0]['terraform']['state']['s3_path'] == '/exampleproject/test/tf.state'

    # test when someone puts in just the s3_path
    config = load_minimal(config_file)
    config['aws-account'][0]['terraform']['state']['s3_path'] = 'override'
    config = process_config(config)
    assert config['aws-account'][0]['terraform']['state']['s3_bucket'] == EXAMPLE_STATE_S3_BUCKET

    # test when someone puts in just the s3_bucket
    config = load_minimal(config_file)
    config['aws-account'][0]['terraform']['state']['s3_path'] = 'override'
    config = process_config(config)
    assert config['aws-account'][0]['terraform']['state']['s3_bucket'] == EXAMPLE_STATE_S3_BUCKET

    # test when someone puts in just the dynamodb table
    config = load_minimal(config_file)
    config['aws-account'][0]['terraform']['state']['s3_path'] = 'override'
    config = process_config(config)
    assert config['aws-account'][0]['terraform']['state']['s3_bucket'] == EXAMPLE_STATE_S3_BUCKET
