"""
These test cases are designed to exercise the launch stage of the
pipeline.
"""
from code.src.summoner.launch import find_last_deploy
from invoke import MockContext, Result


def test_find_last_deploy_exists():
    """
    Test that we can correctly return the largest matching
    tag number from a git show-ref return when tags exist
    """
    tag_list = [
        '2262382e685462c78edc48de81da1daacd6fe679 refs/tags/summoner-stage-1',
        '2262382e685462c78edc48de81da1daacd6fe679 refs/tags/summoner-prod-r11',
        '2262382e685462c78edc48de81da1daacd6fe679 refs/tags/summoner-stage-r2',
        '2262382e685462c78edc48de81da1daacd6fe679 refs/tags/summoner-prod-r3',
        '2262382e685462c78edc48de81da1daacd6fe679 refs/tags/summoner-stage-r30'
    ]
    tag_blob = "\n".join(tag_list)

    mock_results = dict()
    mock_results["git show-ref --tags"] = Result(tag_blob, exited=0)

    stage_version_id = find_last_deploy(MockContext(run=mock_results.copy()), 'summoner-stage-r')
    prod_version_id = find_last_deploy(MockContext(run=mock_results.copy()), 'summoner-prod-r')

    assert stage_version_id == 30
    assert prod_version_id == 11

def test_find_last_deploy_no_tags():
    """
    Test that we can correctly return None when there are
    no matching tags
    """
    mock_results = dict()
    mock_results["git show-ref --tags"] = Result("", exited=0)

    prod_version_id = find_last_deploy(MockContext(run=mock_results.copy()), 'summoner-prod-r')
    assert prod_version_id is None

# def test_find_last_deploy_git_error():
#     """
#     Test the correct exception is raised if Git were
#     to have some kind of error
#     """
#     mock_results = dict()
#     mock_results["git show-ref --tags"] = Result("", exited=1)
#
#     with pytest.raises(RuntimeError) as excinfo:
#         find_last_deploy(MockContext(run=mock_results.copy()), 'summoner-prod-r')
#     assert "Failure in the Git command" in str(excinfo.value)
